#include "ALAudio.h"

#include <mikmod.h>
#include <stdlib.h>

#define MINSIZE  16

#define UNIQUEKEY(x) ((int) (x->Handle))
#define CLEARKEY(us) (us->Handle = (void *) 0)

USoundMap::USoundMap() {
	pool = new HashMM<poolNode_t>(MINSIZE);

	return;
}

USoundMap::~USoundMap() {
	delete pool;

	return;
}

int USoundMap::add(USound *us) {
	static ALboolean (*bufConv)(ALuint, void *, ALboolean, ALuint) = 0;
	poolNode_t *entry;

	if(bufConv == 0) {
		bufConv = (ALboolean (*)(ALuint, void *, ALboolean, ALuint))
			alGetProcAddress((const ALubyte *) "alutBufferAndConvertData_LOKI");

		if(bufConv == 0) {
#ifdef DEBUG_SOUND
			fprintf(stderr, "Could not getproc bufConv\n");
#endif
			return 0;
		}
	}

	entry = new poolNode_t;

	alGenBuffers(1, &entry->bid);
	entry->us = us;

	int err = bufConv(entry->bid, &us->Data(0), AL_FALSE, us->Data.Num());
	if(err == AL_FALSE) {
#ifdef DEBUG_SOUND
		fprintf(stderr, "Could not buffer and convert\n");
#endif

		alDeleteBuffers(1, &entry->bid);

		delete entry;
		return 0;
	}

	pool->add(entry->bid, entry);

	return entry->bid;
}

void USoundMap::remove(USound *us) {
	const poolNode_t *entry = pool->get(UNIQUEKEY(us));
	ALuint bid;

	if(entry == NULL) {
		return;
	}

	bid = entry->bid;

	alDeleteBuffers(1, &bid);

	pool->remove(UNIQUEKEY(us));

	CLEARKEY(us);

	delete entry;

	return;
}

ALuint USoundMap::find(USound *us) {
	const poolNode_t *entry = pool->get(UNIQUEKEY(us));

	if(entry == NULL) {
		return 0;
	}

	return entry->bid;
}
